"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricCollector = void 0;
var ts_morph_1 = require("ts-morph");
var MetricCollector = /** @class */ (function () {
    function MetricCollector() {
        var bodyNodes = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            bodyNodes[_i] = arguments[_i];
        }
        this._numberOfStatements = 0;
        this._cyclomaticComplexity = 1;
        this._modifiedCyclomaticComplexity = 1;
        this._logicalOperations = 0;
        this._maxIndent = 0;
        this._currentIndent = 0;
        this._nodes = [];
        this._nodes = bodyNodes;
    }
    MetricCollector.prototype.collectMetrics = function () {
        for (var _i = 0, _a = this._nodes; _i < _a.length; _i++) {
            var node = _a[_i];
            this.traverse(node);
        }
    };
    Object.defineProperty(MetricCollector.prototype, "numberOfStatements", {
        get: function () {
            return this._numberOfStatements;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MetricCollector.prototype, "cyclomaticComplexity", {
        get: function () {
            return this._cyclomaticComplexity;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MetricCollector.prototype, "modifiedCyclomaticComplexity", {
        get: function () {
            return this._modifiedCyclomaticComplexity;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MetricCollector.prototype, "maxIndent", {
        get: function () {
            return this._maxIndent;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MetricCollector.prototype, "logicalOperations", {
        get: function () {
            return this._logicalOperations;
        },
        enumerable: false,
        configurable: true
    });
    MetricCollector.prototype.indent = function () {
        this._currentIndent++;
        this._maxIndent = Math.max(this._maxIndent, this._currentIndent);
    };
    MetricCollector.prototype.dedent = function () {
        this._currentIndent--;
    };
    MetricCollector.prototype.traverse = function (node) {
        var _this = this;
        switch (node.getKind()) {
            // Skip nested declarations
            case ts_morph_1.SyntaxKind.ClassDeclaration:
            case ts_morph_1.SyntaxKind.ClassExpression:
            case ts_morph_1.SyntaxKind.FunctionExpression:
            case ts_morph_1.SyntaxKind.InterfaceDeclaration:
            case ts_morph_1.SyntaxKind.FunctionDeclaration:
            case ts_morph_1.SyntaxKind.EnumDeclaration:
            case ts_morph_1.SyntaxKind.ArrowFunction:
                return;
        }
        if (node.getKind() >= ts_morph_1.SyntaxKind.FirstStatement && node.getKind() <= ts_morph_1.SyntaxKind.LastStatement) {
            this._numberOfStatements++;
        }
        switch (node.getKind()) {
            case ts_morph_1.SyntaxKind.ForStatement:
            case ts_morph_1.SyntaxKind.ForOfStatement:
            case ts_morph_1.SyntaxKind.ForInStatement:
            case ts_morph_1.SyntaxKind.IfStatement:
            case ts_morph_1.SyntaxKind.WhileStatement:
            case ts_morph_1.SyntaxKind.DoStatement:
                this._cyclomaticComplexity++;
                this._modifiedCyclomaticComplexity++;
                break;
            case ts_morph_1.SyntaxKind.SwitchStatement:
                this._modifiedCyclomaticComplexity++;
                break;
            case ts_morph_1.SyntaxKind.CaseBlock:
                this._cyclomaticComplexity++;
                break;
            case ts_morph_1.SyntaxKind.BinaryExpression:
                var be = node;
                var ot = be.getOperatorToken().getKind();
                if (ot == ts_morph_1.SyntaxKind.AmpersandAmpersandToken || ot == ts_morph_1.SyntaxKind.BarBarToken)
                    this._logicalOperations++;
                break;
        }
        switch (node.getKind()) {
            case ts_morph_1.SyntaxKind.ForStatement:
                var fs = node;
                if (fs.getCondition())
                    this.traverse(fs.getCondition());
                this.indent();
                this.traverse(fs.getStatement());
                this.dedent();
                break;
            case ts_morph_1.SyntaxKind.ForOfStatement:
                var fo = node;
                this.indent();
                this.traverse(fo.getStatement());
                this.dedent();
                break;
            case ts_morph_1.SyntaxKind.ForInStatement:
                var fi = node;
                this.indent();
                this.traverse(fi.getStatement());
                this.dedent();
                break;
            case ts_morph_1.SyntaxKind.IfStatement:
                var ifstmt = node;
                this.traverse(ifstmt.getExpression());
                this.indent();
                this.traverse(ifstmt.getThenStatement());
                this.dedent();
                var es = ifstmt.getElseStatement();
                if (es) {
                    if (es.getKind() == ts_morph_1.SyntaxKind.IfStatement) {
                        // Special treatment for "else if"
                        this.traverse(es);
                    }
                    else {
                        this.indent();
                        this.traverse(es);
                        this.dedent();
                    }
                }
                break;
            case ts_morph_1.SyntaxKind.WhileStatement:
                var ws = node;
                this.traverse(ws.getExpression());
                this.indent();
                this.traverse(ws.getStatement());
                this.dedent();
                break;
            case ts_morph_1.SyntaxKind.DoStatement:
                var ds = node;
                this.traverse(ds.getExpression());
                this.indent();
                this.traverse(ds.getStatement());
                this.dedent();
                break;
            case ts_morph_1.SyntaxKind.SwitchStatement:
                var st = node;
                this.indent();
                st.getClauses().forEach(function (clause) { return _this.traverse(clause); });
                this.dedent();
                break;
            case ts_morph_1.SyntaxKind.TryStatement:
                var ts = node;
                this.indent();
                this.traverse(ts.getTryBlock());
                if (ts.getCatchClause())
                    this.traverse(ts.getCatchClause());
                if (ts.getFinallyBlock())
                    this.traverse((ts.getFinallyBlock()));
                this.dedent();
                break;
            case ts_morph_1.SyntaxKind.WithStatement:
                var wis = node;
                this.indent();
                this.traverse(wis.getStatement());
                this.dedent();
                break;
            default:
                node.forEachChild(function (cn) { return _this.traverse(cn); });
                break;
        }
    };
    return MetricCollector;
}());
exports.MetricCollector = MetricCollector;
//# sourceMappingURL=metric_collector.js.map