"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllElements = exports.lookupNode = exports.lookupSymbol = exports.getElementById = exports.PropertyOrVariable = exports.FunctionOrMethodWithoutBody = exports.FunctionOrMethodWithBody = exports.FunctionOrMethod = exports.Enum = exports.Interface = exports.Class = exports.TypeAlias = exports.SourceFileElement = exports.Namespace = exports.BaseModelElementWithNode = exports.BaseModelElement = exports.Root = exports.Dependency = exports.DependencyKind = exports.ElementType = void 0;
var metric_collector_1 = require("./metric_collector");
var ElementType;
(function (ElementType) {
    ElementType[ElementType["Root"] = 0] = "Root";
    ElementType[ElementType["SourceFile"] = 1] = "SourceFile";
    ElementType[ElementType["Function"] = 2] = "Function";
    ElementType[ElementType["Constructor"] = 3] = "Constructor";
    ElementType[ElementType["Method"] = 4] = "Method";
    ElementType[ElementType["Variable"] = 5] = "Variable";
    ElementType[ElementType["Property"] = 6] = "Property";
    ElementType[ElementType["Getter"] = 7] = "Getter";
    ElementType[ElementType["Setter"] = 8] = "Setter";
    ElementType[ElementType["Class"] = 9] = "Class";
    ElementType[ElementType["Interface"] = 10] = "Interface";
    ElementType[ElementType["Enum"] = 11] = "Enum";
    ElementType[ElementType["EnumMember"] = 12] = "EnumMember";
    ElementType[ElementType["Namespace"] = 13] = "Namespace";
    ElementType[ElementType["TypeAlias"] = 14] = "TypeAlias";
})(ElementType || (exports.ElementType = ElementType = {}));
var DependencyKind;
(function (DependencyKind) {
    DependencyKind[DependencyKind["READ"] = 0] = "READ";
    DependencyKind[DependencyKind["WRITE"] = 1] = "WRITE";
    DependencyKind[DependencyKind["CALL"] = 2] = "CALL";
    DependencyKind[DependencyKind["NEW"] = 3] = "NEW";
    DependencyKind[DependencyKind["EXTENDS"] = 4] = "EXTENDS";
    DependencyKind[DependencyKind["IMPLEMENTS"] = 5] = "IMPLEMENTS";
    DependencyKind[DependencyKind["USES"] = 6] = "USES";
})(DependencyKind || (exports.DependencyKind = DependencyKind = {}));
var nextId = 2;
var elements = [];
var symbolTable = new Map();
var alternateSymbolTable = new Map();
var Dependency = /** @class */ (function () {
    function Dependency(fromId, toId, line, kind) {
        this.fromId = fromId;
        this.toId = toId;
        this.line = line;
        this.kind = kind;
    }
    Dependency.prototype.toJson = function () {
        return [this.fromId, this.toId, this.line, DependencyKind[this.kind]];
    };
    return Dependency;
}());
exports.Dependency = Dependency;
var Root = /** @class */ (function () {
    function Root(tsConfigPath) {
        this.children = [];
        this.tsConfigPath = tsConfigPath;
        elements.push(this);
        this.dependencies = [];
    }
    Object.defineProperty(Root.prototype, "name", {
        get: function () {
            return "Root";
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Root.prototype, "type", {
        get: function () {
            return ElementType.Root;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Root.prototype, "parent", {
        get: function () {
            return null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Root.prototype, "node", {
        get: function () {
            return null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Root.prototype, "line", {
        get: function () {
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Root.prototype, "id", {
        get: function () {
            return 1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Root.prototype, "sourceFile", {
        get: function () {
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Root.prototype, "isExternal", {
        get: function () {
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    Root.prototype.addDependency = function (from, to, line, kind) {
        this.dependencies.push(new Dependency(from.id, to.id, line, kind));
    };
    Root.prototype.toJson = function () {
        return { tsConfigPath: this.tsConfigPath, "numberOfElements": nextId - 2, children: this.children.map(function (val, index) { return val.toJson(); }), dependencies: this.dependencies.map(function (val, index) { return val.toJson(); }) };
    };
    return Root;
}());
exports.Root = Root;
var BaseModelElement = /** @class */ (function () {
    function BaseModelElement(parent, name) {
        this.parent = parent;
        this.name = name;
        this.children = [];
        parent.children.push(this);
        this.id = nextId++;
        elements.push(this);
    }
    Object.defineProperty(BaseModelElement.prototype, "sourceFile", {
        get: function () {
            return this.parent.sourceFile;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseModelElement.prototype, "isExternal", {
        get: function () {
            return this.parent.isExternal;
        },
        enumerable: false,
        configurable: true
    });
    BaseModelElement.prototype.toJson = function () {
        return { idNameKind: [this.id, this.name, ElementType[this.type]], children: this.children.map(function (value, index) { return value.toJson(); }) };
    };
    return BaseModelElement;
}());
exports.BaseModelElement = BaseModelElement;
var BaseModelElementWithNode = /** @class */ (function (_super) {
    __extends(BaseModelElementWithNode, _super);
    function BaseModelElementWithNode(parent, name, node, line) {
        var _this = _super.call(this, parent, name) || this;
        _this.name = name;
        _this.node = node;
        _this.line = line;
        var sym = node.getSymbol();
        var compilerNode = node.compilerNode;
        if (sym) {
            symbolTable.set(sym, _this);
        }
        alternateSymbolTable.set(compilerNode, _this);
        return _this;
    }
    BaseModelElementWithNode.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.line = this.line;
        return result;
    };
    return BaseModelElementWithNode;
}(BaseModelElement));
exports.BaseModelElementWithNode = BaseModelElementWithNode;
var Namespace = /** @class */ (function (_super) {
    __extends(Namespace, _super);
    function Namespace(parent, name, node, line) {
        return _super.call(this, parent, name, node, line) || this;
    }
    Object.defineProperty(Namespace.prototype, "type", {
        get: function () {
            return ElementType.Namespace;
        },
        enumerable: false,
        configurable: true
    });
    return Namespace;
}(BaseModelElementWithNode));
exports.Namespace = Namespace;
var SourceFileElement = /** @class */ (function (_super) {
    __extends(SourceFileElement, _super);
    function SourceFileElement(parent, name, node, isExternal) {
        if (isExternal === void 0) { isExternal = false; }
        var _this = _super.call(this, parent, name) || this;
        _this.node = node;
        _this.numberOfStatements = -1;
        _this.cyclomaticComplexity = -1;
        _this.modifiedCyclomaticComplexity = -1;
        _this.logicalOperations = -1;
        _this.maxIndent = -1;
        _this._isExternal = isExternal;
        return _this;
    }
    Object.defineProperty(SourceFileElement.prototype, "type", {
        get: function () {
            return ElementType.SourceFile;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SourceFileElement.prototype, "line", {
        get: function () {
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SourceFileElement.prototype, "sourceFile", {
        get: function () {
            return this.name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SourceFileElement.prototype, "isExternal", {
        get: function () {
            return this._isExternal;
        },
        enumerable: false,
        configurable: true
    });
    SourceFileElement.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.isExternal = this._isExternal;
        if (this.numberOfStatements >= 0)
            result.bodyMetrics = [this.numberOfStatements, this.cyclomaticComplexity, this.modifiedCyclomaticComplexity, this.logicalOperations, this.maxIndent];
        return result;
    };
    return SourceFileElement;
}(BaseModelElement));
exports.SourceFileElement = SourceFileElement;
var TypeAlias = /** @class */ (function (_super) {
    __extends(TypeAlias, _super);
    function TypeAlias(parent, name, node, line) {
        var _this = _super.call(this, parent, name, node, line) || this;
        _this.typeParameters = "";
        return _this;
    }
    Object.defineProperty(TypeAlias.prototype, "type", {
        get: function () {
            return ElementType.TypeAlias;
        },
        enumerable: false,
        configurable: true
    });
    TypeAlias.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.typeParameters = this.typeParameters;
        return result;
    };
    return TypeAlias;
}(BaseModelElementWithNode));
exports.TypeAlias = TypeAlias;
var Class = /** @class */ (function (_super) {
    __extends(Class, _super);
    function Class(parent, name, node, line, isAbstract) {
        var _this = _super.call(this, parent, name, node, line) || this;
        _this.isAbstract = isAbstract;
        _this.typeParameters = "";
        return _this;
    }
    Object.defineProperty(Class.prototype, "type", {
        get: function () {
            return ElementType.Class;
        },
        enumerable: false,
        configurable: true
    });
    Class.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.isAbstract = this.isAbstract;
        result.typeParameters = this.typeParameters;
        return result;
    };
    return Class;
}(BaseModelElementWithNode));
exports.Class = Class;
var Interface = /** @class */ (function (_super) {
    __extends(Interface, _super);
    function Interface(parent, name, node, line) {
        var _this = _super.call(this, parent, name, node, line) || this;
        _this.typeParameters = "";
        return _this;
    }
    Object.defineProperty(Interface.prototype, "type", {
        get: function () {
            return ElementType.Interface;
        },
        enumerable: false,
        configurable: true
    });
    Interface.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.typeParameters = this.typeParameters;
        return result;
    };
    return Interface;
}(BaseModelElementWithNode));
exports.Interface = Interface;
var Enum = /** @class */ (function (_super) {
    __extends(Enum, _super);
    function Enum(parent, name, node, line) {
        return _super.call(this, parent, name, node, line) || this;
    }
    Object.defineProperty(Enum.prototype, "type", {
        get: function () {
            return ElementType.Enum;
        },
        enumerable: false,
        configurable: true
    });
    return Enum;
}(BaseModelElementWithNode));
exports.Enum = Enum;
var FunctionOrMethod = /** @class */ (function (_super) {
    __extends(FunctionOrMethod, _super);
    function FunctionOrMethod(parent, name, node, line) {
        var _this = _super.call(this, parent, name, node, line) || this;
        _this.numberOfParameters = 0;
        _this.parameters = "";
        _this.typeParameters = "";
        _this.returnType = "";
        return _this;
    }
    FunctionOrMethod.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.numberOfParameters = this.numberOfParameters;
        result.isAbstract = this.isAbstract;
        result.parameters = this.parameters;
        result.typeParameters = this.typeParameters;
        result.returnType = this.returnType;
        return result;
    };
    return FunctionOrMethod;
}(BaseModelElementWithNode));
exports.FunctionOrMethod = FunctionOrMethod;
var FunctionOrMethodWithBody = /** @class */ (function (_super) {
    __extends(FunctionOrMethodWithBody, _super);
    function FunctionOrMethodWithBody(parent, name, node, line, type, bodyNode) {
        var _this = _super.call(this, parent, name, node, line) || this;
        _this.type = type;
        _this.bodyNode = bodyNode;
        _this.numberOfStatements = -1;
        _this.cyclomaticComplexity = -1;
        _this.modifiedCyclomaticComplexity = -1;
        _this.logicalOperations = -1;
        _this.maxIndent = -1;
        var mc = new metric_collector_1.MetricCollector(_this.bodyNode);
        mc.collectMetrics();
        _this.numberOfStatements = mc.numberOfStatements;
        _this.cyclomaticComplexity = mc.cyclomaticComplexity;
        _this.modifiedCyclomaticComplexity = mc.modifiedCyclomaticComplexity;
        _this.logicalOperations = mc.logicalOperations;
        _this.maxIndent = mc.maxIndent;
        return _this;
    }
    Object.defineProperty(FunctionOrMethodWithBody.prototype, "isAbstract", {
        get: function () {
            return false;
        },
        enumerable: false,
        configurable: true
    });
    FunctionOrMethodWithBody.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.hasBody = true;
        result.bodyMetrics = [this.numberOfStatements, this.cyclomaticComplexity, this.modifiedCyclomaticComplexity, this.logicalOperations, this.maxIndent];
        return result;
    };
    return FunctionOrMethodWithBody;
}(FunctionOrMethod));
exports.FunctionOrMethodWithBody = FunctionOrMethodWithBody;
var FunctionOrMethodWithoutBody = /** @class */ (function (_super) {
    __extends(FunctionOrMethodWithoutBody, _super);
    // type can be Method, Getter or Setter
    function FunctionOrMethodWithoutBody(parent, name, node, line, type, isAbstract) {
        var _this = _super.call(this, parent, name, node, line) || this;
        _this.type = type;
        _this.isAbstract = isAbstract;
        return _this;
    }
    FunctionOrMethodWithoutBody.prototype.toJson = function () {
        var result = _super.prototype.toJson.call(this);
        result.hasBody = false;
        return result;
    };
    return FunctionOrMethodWithoutBody;
}(FunctionOrMethod));
exports.FunctionOrMethodWithoutBody = FunctionOrMethodWithoutBody;
var PropertyOrVariable = /** @class */ (function (_super) {
    __extends(PropertyOrVariable, _super);
    function PropertyOrVariable(parent, name, node, line, type, initializer) {
        var _this = _super.call(this, parent, name, node, line) || this;
        _this.type = type;
        _this.initializer = initializer;
        return _this;
    }
    return PropertyOrVariable;
}(BaseModelElementWithNode));
exports.PropertyOrVariable = PropertyOrVariable;
function getElementById(id) {
    return elements[id - 1];
}
exports.getElementById = getElementById;
function lookupSymbol(symbol) {
    return symbolTable.get(symbol);
}
exports.lookupSymbol = lookupSymbol;
function lookupNode(node) {
    return alternateSymbolTable.get(node.compilerNode);
}
exports.lookupNode = lookupNode;
function getAllElements() {
    return elements;
}
exports.getAllElements = getAllElements;
//# sourceMappingURL=model.js.map